#!/usr/bin/python3

# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of Yaji.
#
# Yaji is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yaji is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

import ___sampleapp  # this is just an internal hack - just ignore it and don't copy it!

import time

import yaji


class Application(yaji.Application):

    def __init__(self):
        super().__init__(head1="Yaji Sample", head2="curtain", stop_implicitly_when_browser_closed=True)

        # stuff for 'data bindings'
        h00 = self.create_datastore(curtain='mycurtain0')
        h01 = self.create_datastore(curtain='mycurtain0')
        h0e = self.create_datastore(curtain='mycurtain0')
        def h0changed(_):
            time.sleep(2)  # simulate some calculation
            h0e.setvalue(h00.getvalue() == "foo" and h01.getvalue() == "bar")
        h00.add_onchanged_handler(h0changed)
        h01.add_onchanged_handler(h0changed)

        # stuff for 'ajax requests' (more below)
        self.vh10 = self.vh11 = None
        self.h1e = self.create_datastore()
        self._refresh_h1e()

        # stuff for 'with default curtain'
        h20 = self.create_datastore(curtain=True)

        self.setbodyleft(yaji.Grid(  # the buttons don't actually do something
            yaji.View(label="data bindings", style="font-size:1.4em", row=0),
            yaji.View('Label', row=1, col=0, label="Enter 'foo' here:"),
            yaji.View('EditBox', row=1, col=1, text=self.binddata(h00)),
            yaji.View('Label', row=2, col=0, label="Enter 'bar' here:"),
            yaji.View('EditBox', row=2, col=1, text=self.binddata(h01)),
            yaji.View('Button', label="Go", row=3, col=1, enabled=self.binddata(h0e), name='btn0'),

            yaji.View(label="ajax requests", style="font-size:1.4em", row=4),
            yaji.View('Label', row=5, col=0, label="Enter 'foo' here:"),
            yaji.View('EditBox', row=5, col=1, name='eh10'),
            yaji.View('Label', row=6, col=0, label="Enter 'bar' here:"),
            yaji.View('EditBox', row=6, col=1, name='eh11'),
            yaji.View('Button', label="Go", row=7, col=1, enabled=self.binddata(self.h1e), name='btn1'),

            yaji.View(label="with default curtain", style="font-size:1.4em", row=8),
            yaji.View('CheckButton', row=9, col=1, checked=self.binddata(h20)),
            yaji.View('CheckButton', row=10, col=1, checked=self.binddata(h20)),

        ))

    @yaji.RequestHandler.for_urls(auto=True)
    def _do_set_h10(self, v: str):
        self.vh10 = v
        self._refresh_h1e()

    @yaji.RequestHandler.for_urls(auto=True)
    def _do_set_h11(self, v: str):
        self.vh11 = v
        self._refresh_h1e()

    def _refresh_h1e(self):
        time.sleep(2)  # simulate some calculation
        self.h1e.setvalue(self.vh10 == "foo" and self.vh11 == "bar")


if __name__ == '__main__':
    Application().start()


#TODO cooler default curtain, playing with focussing
