#!/usr/bin/python3

# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of Yaji.
#
# Yaji is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yaji is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Yaji.  If not, see <http://www.gnu.org/licenses/>.

import ___sampleapp  # this is just an internal hack - just ignore it and don't copy it!

import yaji


class Application(yaji.Application):

    def __init__(self):
        super().__init__(head1="Yaji Sample", head2="middleware", stop_implicitly_when_browser_closed=True)
        self.add_middleware(Application.MyMiddleware("Andreas", "Christian"))
        self.add_middleware(Application.MyMiddleware("Stefan", "Kilian"))
        self.add_middleware(Application.MyMiddleware("Kilian", "Andreas"))
        self.add_middleware(Application.MyEarlyMiddleware("Markus", "Stefan"))
        self.setbodyleft(yaji.View('Label', label="Der Andreas."))  # should end up in 'Christian'
        self.setbodyright(yaji.View('Label', label="Der Markus."))  # should end up in 'Andreas'

    class MyMiddleware(yaji.Middleware):

        def __init__(self, replace: str, replaceby: str, **b):
            super().__init__(**b)
            self.__replace = replace.encode()
            self.__replaceby = replaceby.encode()

        @yaji.Middleware.request_processor()
        def _replacestuff(self, request):
            if isinstance(request.response_body, bytes):
                request.response_body = request.response_body.replace(self.__replace, self.__replaceby)

    class MyEarlyMiddleware(yaji.Middleware):  # same as MyMiddleware, but with `index=-1`

        def __init__(self, replace: str, replaceby: str, **b):
            super().__init__(**b)
            self.__replace = replace.encode()
            self.__replaceby = replaceby.encode()

        @yaji.Middleware.request_processor(index=-1)
        def _replacestuff(self, request):
            if isinstance(request.response_body, bytes):
                request.response_body = request.response_body.replace(self.__replace, self.__replaceby)


if __name__ == '__main__':
    Application().start()
