/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2014-2020, Josef Hahn

 This file is part of anise.

 anise is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 anise is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with anise.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

class _WebConsoleTreeDatasource_PullSource {

    constructor(node, requesttoken) {
        this.node = node;
        this.nodesortkey = node.getMetadata("sortkey");
        this.requesttoken = requesttoken;
    }

    static pullSources(sources) {
        if (sources.length > 0) {
            var s1 = sources[0];
            sources.splice(0, 1);
            s1.pull(function(){
                _WebConsoleTreeDatasource_PullSource.pullSources(sources);
            });
        }
    }

    _islower(item, inspos) {
        var cnode = this.node.childs[inspos][0];
        var cnodekey = cnode.getMetadata().sortkey;
        if (item.name == "internals")
            return true;
        else if (cnodekey < this.sortkey)
            return true;
        else if (cnodekey > this.sortkey)
            return false;
        else
            return cnode.value < item.name;
    }

    pull(ondone) {
        var self = this;
        yaji.ajax({
            url: this.sourcename,
            data: {"p": this.node.path},
            onsuccess: function(data) {
                if (self.requesttoken != self.node.lastrequest)
                    return;
                var xxx = clove.utils.suspendResizing();
                try {
                    var curlst = [];
                    for(var ich=0; ich<self.node.childs.length; ich++) {
                        var chn = self.node.childs[ich][0];
                        if (chn.getMetadata().pullsourcename == self.sourcename)
                            curlst.push(chn);
                    }
                    for(var i in data.items) {
                        var mitem = data.items[i];
                        var docreate = true;
                        for(var ich=0; ich<curlst.length; ich++) {
                            if (curlst[ich].value == mitem.name) {
                                curlst.splice(ich, 1);
                                docreate = false;
                                break;
                            }
                        }
                        if (docreate) {
                            var inspos = 0;
                            while (inspos < self.node.childs.length && self._islower(mitem, inspos))
                                inspos++;
                            var n = self.node.insertRow(inspos, [mitem.name]);
                            if (mitem.iconhint)
                                n.setMetadata("icon", clove.Icon.byUrl("static/icons/" + mitem.iconhint));
                            n.setMetadata("pullsourcename", self.sourcename);
                            n.setMetadata("sortkey", self.sortkey);
                        }
                    }
                    for(var ich=curlst.length-1; ich>=0; ich--)
                        self.node.removeRow(curlst[ich].ir);
                }
                finally {
                    clove.utils.resumeResizing(xxx);
                }
            },
            onfinished: ondone,
        });
    }

}

class _WebConsoleTreeDatasource_PullSource_Features extends _WebConsoleTreeDatasource_PullSource {

    constructor(node, requesttoken) {
        super(node, requesttoken);
        this.sourcename = "listfeatures";
        this.sortkey = 0;
    }

}

class _WebConsoleTreeDatasource_PullSource_Keys extends _WebConsoleTreeDatasource_PullSource {

    constructor(node, requesttoken) {
        super(node, requesttoken);
        this.sourcename = "listkeys";
        this.sortkey = 1;
    }

}

class _WebConsoleTreeDatasource_PullSource_Members extends _WebConsoleTreeDatasource_PullSource {

    constructor(node, requesttoken) {
        super(node, requesttoken);
        this.sourcename = "listmembers";
        this.sortkey = 2;
    }

}

class WebConsoleTreeDatasource extends clove.AsyncDatasource {

    constructor() {
        super();
        this.universe = this.root().addRow(["universe"]);
        this.universe.path = "universe";
    }

    do_async_pull(ptr, requestConfig) {
        if (ptr) {
            var node = this.valuePointerToNativeNode(ptr);
            var self = this;
            var myrequest = node.lastrequest = new Object();
            _WebConsoleTreeDatasource_PullSource.pullSources([
                 new _WebConsoleTreeDatasource_PullSource_Features(node, myrequest),
                 new _WebConsoleTreeDatasource_PullSource_Keys(node, myrequest),
                 new _WebConsoleTreeDatasource_PullSource_Members(node, myrequest),
            ]);
        }
    }

    _getNode(ir, ic, parent) {
        var result = super._getNode(ir, ic, parent);
        if (!result.path && result.value)
            result.path = this.valuePointerToNativeNode(parent).path
                    + (result.value.startsWith("[") ? "" : ".")
                    + result.value;
        return result;
    }

}

class WebConsoleTree extends clove.TreeView {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            allowSelection: true,
        }), domnode);
    }

    doinit() {
        super.doinit();
        var ds = new WebConsoleTreeDatasource();
        this.setDatasource(ds);
        ds.universe.setMetadata("icon", clove.Icon.byUrl("static/icons/pde/universe.png"));
    }

}

class WebConsolePanel extends clove.HtmlView {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
                styleClass: 'anise_webconsolepanel_body',
                width: '1pt',
                height: '1pt',
                vstretch: 1,
                hstretch: 1,
        }), domnode);
        this.declareProperty("item");
        this.addStyleClass("anise_webconsolepanel");
        this.OnSetPrompt = new clove.Event();
        this.OnActionExecuted = new clove.Event();
        this._nextterminalconstructindex = 0;
    }

    item_setter(v) {
        var self = this;
        var mainpanel = this.contentnode;
        mainpanel.innerHTML = "";
        if (v) {
            var currentrequest = this.request = new Object();
            yaji.ajax({
                url: "info",
                data: {"p": v},
                onsuccess: function(data) {
                    function _createheader(hdtxt) {
                        var r = document.createElement("div");
                        r.className = "clovestyle_text_head3";
                        r.textContent = hdtxt;
                        return r;
                    }
                    if (currentrequest != self.request)
                        return;
                    if (data.docstring)
                    {
                        var mdiv = document.createElement("div");
                        mdiv.className = "docstring";
                        mdiv.textContent = data.docstring;
                        mainpanel.appendChild(mdiv);
                    }
                    if (data.comment)
                    {
                        var h3a = _createheader("Comment");
                        mainpanel.appendChild(h3a);
                        var mdiv = document.createElement("div");
                        mdiv.textContent = data.comment;
                        mainpanel.appendChild(mdiv);
                    }
                    if (data.typeinfo)
                    {
                        var h3b = _createheader("Type");
                        mainpanel.appendChild(h3b);
                        var mdiv = document.createElement("div");
                        mdiv.textContent = data.typeinfo;
                        mainpanel.appendChild(mdiv);
                    }
                    if (data.value)
                    {
                        var h3c = _createheader("Value");
                        mainpanel.appendChild(h3c);
                        var mdiv = document.createElement("div");
                        mdiv.textContent = data.value;
                        mainpanel.appendChild(mdiv);
                        var h3d = _createheader("Available actions");
                        mainpanel.appendChild(h3d);
                        var toterma = document.createElement("a");
                        toterma.className = "anise_webconsolepanel_bodylink";
                        toterma.textContent = "Retrieve in terminal";
                        var yul = document.createElement("ul");
                        var yli = document.createElement("li");
                        mainpanel.appendChild(yul);
                        yul.appendChild(yli);
                        yli.appendChild(toterma);
                        toterma.addEventListener("click", function(e){
                            e.preventDefault();
                            self.OnSetPrompt.trigger({widget: self, prompt: v});
                        }, false);

                    }
                    if (data.arguments)
                    {
                        var h3d = _createheader("Arguments");
                        mainpanel.appendChild(h3d);
                        var terminalargspart = "";
                        if (data.arguments.args.length>0)
                        {
                            var argul = document.createElement("ul");
                            mainpanel.appendChild(argul);
                            for(var i=0;i<data.arguments.args.length;i++)
                            {
                                if ((data.isbound||data.isclass) && i==0)
                                    continue;
                                var member = data.arguments.args[i];
                                var name = member[0];
                                var hasdefault = member[1];
                                var vdefault = member[2];
                                var onlykw = member[3];
                                var argli = document.createElement("li");
                                var argliname = document.createElement("div");
                                var arglinotes = document.createElement("div");
                                var arglidoc = document.createElement("div");
                                argliname.className = "argumentname";
                                arglinotes.className = "argumentnotes";
                                arglidoc.className = "argumentdocumentation";
                                argli.appendChild(argliname);
                                argli.appendChild(arglinotes);
                                argli.appendChild(arglidoc);
                                argliname.textContent = member[0];
                                if (hasdefault)
                                    arglinotes.textContent += " (default: " + vdefault + ")";
                                if (onlykw)
                                    arglinotes.textContent += " (only allowed as keyword argument)";
                                if (data.arguments.doc)
                                {
                                    var argdoc = data.arguments.doc[name];
                                    if (argdoc)
                                        arglidoc.textContent = argdoc;
                                }
                                argul.appendChild(argli);
                                if (!hasdefault && (!(data.isclass||data.isbound) || i>0))
                                {
                                    if (terminalargspart)
                                        terminalargspart += ", ";
                                    terminalargspart += name + "=None";
                                }
                            }
                        }
                        else
                        {
                            var noargs = document.createElement("div");
                            noargs.className = "argumentsnotes";
                            noargs.textContent = "(none)";
                            mainpanel.appendChild(noargs);
                        }
                        if (data.arguments.varargs)
                        {
                            var mdiv = document.createElement("div");
                            mdiv.className = "argumentsnotes";
                            mdiv.textContent = "(allows additional positional arguments)";
                            mainpanel.appendChild(mdiv);
                        }
                        if (data.arguments.varkw)
                        {
                            var mdiv = document.createElement("div");
                            mdiv.className = "argumentsnotes";
                            mdiv.textContent = "(allows additional keyword arguments)";
                            mainpanel.appendChild(mdiv);
                        }
                        if (data.returns)
                        {
                            var h3d = _createheader("Returns");
                            mainpanel.appendChild(h3d);
                            var argul = document.createElement("div");
                            argul.className = "argumentdocumentation returnsdocumentation";
                            argul.textContent = data.returns;
                            mainpanel.appendChild(argul);
                        }
                        var h3d = _createheader("Available actions");
                        mainpanel.appendChild(h3d);
                        var toterma = document.createElement("a");
                        toterma.className = "anise_webconsolepanel_bodylink";
                        toterma.textContent = data.isclass ? "Construct in terminal" : "Execute in terminal";
                        var yul = document.createElement("ul");
                        var yli = document.createElement("li");
                        mainpanel.appendChild(yul);
                        yul.appendChild(yli);
                        yli.appendChild(toterma);
                        toterma.addEventListener("click", function(e){
                            e.preventDefault();
                            var line = v + "(" + terminalargspart + ")";
                            if (data.isclass) {
                                var myindex = self._nextterminalconstructindex++;
                                var myindexstring = "";
                                while (myindex >= 0)
                                {
                                    var r = (myindex % 26);
                                    myindex = (myindex / 26);
                                    if (r)
                                        myindex = Math.floor(myindex);
                                    myindexstring = String.fromCharCode(r+65) + myindexstring;
                                    if (myindex == 0)
                                        break;
                                }
                                line = "foo" + myindexstring + " = " + line;
                            }
                            self.OnSetPrompt.trigger({widget: self, prompt: line});
                        }, false);
                    }
                    if (data.actions.length > 0)
                    {
                        var h3e = _createheader("Available actions");
                        var acul = document.createElement("ul");
                        acul.className = "actions";
                        mainpanel.appendChild(h3e);
                        mainpanel.appendChild(acul);
                        for(var iaction in data.actions)
                        {
                            var action = data.actions[iaction];
                            var acli = document.createElement("li");
                            var aclia = document.createElement("a");
                            aclia.className = "anise_webconsolepanel_bodylink";
                            aclia.textContent = action.label;
                            var aclidesc = document.createElement("span");
                            if (action.description)
                                aclidesc.textContent = " : " + action.description
                            acli.appendChild(aclia);
                            acli.appendChild(aclidesc);
                            acul.appendChild(acli);
                            function createclickhandler(idx) {
                                return function(e){
                                    e.preventDefault();
                                    yaji.ajax({
                                        url: "execaction",
                                        data: {p: v, idx: idx},
                                        onsuccess: function(data){
                                            self.OnActionExecuted.trigger({widget: self});
                                        }
                                    });
                                }
                            };
                            aclia.addEventListener("click", createclickhandler(action.idx), false);
                        }
                    }
                    mainpanel.appendChild(document.createElement("br"));
                    mainpanel.appendChild(document.createElement("br"));
                    self.relayout();
                }
            });
        }
    }

}

class WebConsolePrompt extends clove.NameScope(clove.StackLayout(clove.Widget)) {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            orientation: 'horizontal',
            children: [
                {label: '⟫', styleClass: 'anise_prompt_prelabel'},
                {view: 'EditBox', hstretch: 1, name: 'txt', styleClass: 'anise_prompt_txtbox'},
                {view: 'Button', styleClass: 'anise_prompt_btn', icon: clove.Icon.bySymbol("⏵"), label: '', name: 'btn', OnClicked: function(e){
                    e.widget.containingNameScope().go();
                }},
            ],
        }), domnode);
        this.addStyleClass("anise_prompt");
        this.history = [];
        this.historyindex = -1;
        this.nextterminalconstructindex = -1;
    }

    setInput(line) {
        var txtbox = this.getByName('txt');
        var btn = this.getByName('btn');
        txtbox.setText(line);
        txtbox.focus();
        txtbox.setTextSelection(line.length, line.length);
        clove.utils.animateNode(this.topnode, "aniseanimation_terminalpromptanimation", 1000);
    }

    execute(command) {
        yaji.ajax({
            url: "exec",
            data: {"p": command},
        });
    }

    go() {
        var txtbox = this.getByName('txt');
        var txt = txtbox.text();
        this.history.push(txt);
        this.execute(txt);
        this.historyindex = -1;
        txtbox.setText("");
    }

    navInHistory(direction) {
        if (this.history.length > 0)
        {
            if (direction == 1)
            {
                if (this.historyindex < this.history.length - 1)
                    this.historyindex++;
            }
            else if (direction == -1)
            {
                if (this.historyindex == -1)
                    this.historyindex = this.history.length - 1;
                else if (this.historyindex > 0)
                    this.historyindex--;
            }
        }
        if (this.historyindex > -1)
            this.getByName('txt').setText(this.history[this.historyindex]);
    }

    doinit() {
        super.doinit();
        var self = this;
        this.getByName('txt').OnKeyDown.addHandler(function(e){
            if (e.keyCode == 10 || e.keyCode == 13)
                self.go();
            else if (e.keyCode == 38)
                self.navInHistory(-1);
            else if (e.keyCode == 40)
                self.navInHistory(1);
        });
    }

}

yaji.ready(function(){

    // build UI

    var universetree = clove.getByName('universetree');
    var panel = clove.getByName('panel');
    var pagerprompt = clove.getByName('pagerprompt');
    universetree.expandCell(universetree.datasource().universe.valuePointer());

    // when selection in tree changes ...
    universetree.OnSelectionChanged.addHandler(function(e){
        var p = universetree.datasource().valuePointerToNativeNode(universetree.selection()).path;
        yaji.mainview.setHead2(p);
        panel.setItem(p);
    });

    // select root node in the tree
    universetree.selectCell(universetree.datasource().universe.valuePointer());

    // when the panel wants to set a line to the prompt ...
    panel.OnSetPrompt.addHandler(function(e){
        pagerprompt.setInput(e.prompt);
    });

    // when the panel has executed an action ...
    panel.OnActionExecuted.addHandler(function(e){
        universetree.OnSelectionChanged.trigger(); //refreshing the ui
    });

});
