"use strict";

class AniseDoc_ImageCarousel {

    _optdims(n) {
        return n._getoptimaldims ? n._getoptimaldims() : { width: n.offsetWidth, height: n.offsetHeight };
    }

    _resize() {
        var mheight = 0;
        var mwidth = 0;
        for(var i=0; i<this.imagenodes.length; i++)
            mheight = Math.max(mheight, this._optdims(this.imagenodes[i]).height);
        var mwidth = this.rootnode.offsetWidth;
        for(var i=0; i<this.imagenodes.length; i++) {
            var inode = this.imagenodes[i];
            inode.style.position = "absolute";
            var optdims = this._optdims(inode);
            inode.style.top = ((mheight - optdims.height) / 2) + "px";
            inode.style.left = ((mwidth - optdims.width) / 2) + "px";
        }
        this.rootnode.style.height = (mheight + this.carousel.offsetHeight) + "px";
    }

    showitem(no, _skipresetswitch) {
        var self = this;
        if (no == this.currentitem)
            return;
        for(var i=0; i<this.imagenodes.length; i++) {
            var inode = this.imagenodes[i];
            if (i == no) {
                inode.style.opacity = 1;
                inode.style.zIndex = 1;
                inode.ilnk.className = inode.ilnk.origClassName + " current";
            }
            else {
                inode.style.opacity = 0;
                inode.style.zIndex = 0;
                inode.ilnk.className = inode.ilnk.origClassName;
            }
        }
        if (this.currentitem !== undefined) {
            var cn = this.imagenodes[this.currentitem];
            if (cn._stop)
                cn._stop();
        }
        this.currentitem = no;
        if (!_skipresetswitch)
            this._resetSwitchNextInterval();
    }

    constructor(rootnode) {
        var self = this;
        this.rootnode = rootnode;
        this.imagenodes = $(rootnode).children();
        this.carousel = document.createElement("div");
        this.carousel.className = "carousel";
        if (this.imagenodes.length < 2)
            return;
        this.rootnode.style.position = "relative";
        this.rootnode.appendChild(this.carousel);
        for(var i=0; i<this.imagenodes.length; i++) {
            var inode = this.imagenodes[i];
            if (inode._registerloadhandler)
                inode._registerloadhandler(function(){ self._resize(); });
            inode.className += " childitem";
            var ilnk = document.createElement("span");
            ilnk.textContent = "⑩";
            if (i % 10 == 9)
                ilnk.className = "fullten";
            ilnk.onclick = (function(ii) { return function(e) { self.showitem(ii); }; })(i);
            this.carousel.appendChild(ilnk);
            inode.ilnk = ilnk;
            ilnk.origClassName = ilnk.className;
        }
        var nxtlnk = document.createElement("span");
        nxtlnk.textContent = "⮩";
        nxtlnk.className = "next";
        nxtlnk.onclick = function() { self.nextitem(); };
        this.carousel.appendChild(nxtlnk);
        var oonresize = window.onresize;
        window.onresize = function() {
            self._resize();
            if (oonresize)
                oonresize();
        }
        this._resize();
        this.showitem(0, true);
        this._initintid = setInterval(function(){
            var r = rootnode.getBoundingClientRect();
            if (r.bottom > 0 && r.top < window.innerHeight) {
                clearInterval(self._initintid);
                self._resetSwitchNextInterval();
            }
        }, 2000);
        rootnode.addEventListener('touchstart', function(event) {
            self._touchbeginpos = undefined;
            var ino = self.imagenodes[self.currentitem];
            if (ino._isactive && ino._isactive())
                return;
            if (event.changedTouches.length == 1) {
                self._touchbeginpos = event.changedTouches[0];
                self._touchbegintime = Date.now();
            }
        }, false);
        rootnode.addEventListener('touchend', function(event) {
            if (self._touchbeginpos) {
                if (Date.now()-self._touchbegintime < 600 && event.changedTouches.length == 1) {
                    var pos1 = self._touchbeginpos;
                    var pos2 = event.changedTouches[0];
                    var dx = pos2.clientX - pos1.clientX;
                    var dy = pos2.clientY - pos1.clientY;
                    var adx = Math.abs(dx);
                    var ady = Math.abs(dy);
                    if (ady < 60 && adx > 40 && adx > ady)
                        self.nextitem(dx > 0);
                }
                self._touchbeginpos = undefined;
            }
        }, false);
    }

    _resetSwitchNextInterval() {
        var self = this;
        if (this._switchnextintid !== undefined)
            clearInterval(this._switchnextintid);
        this._switchnextintid = setInterval(function() {
            var ino = self.imagenodes[self.currentitem];
            if (!ino._isactive || !ino._isactive())
                self.nextitem();
        }, 9000);
    }

    nextitem(prev) {
        this.showitem((this.currentitem+(prev?-1:1)+this.imagenodes.length) % this.imagenodes.length);
    }

}


$(document).ready(function() {
    var banner = $("#bannerimage")[0]; // this will fail if no banner image is there; no real problem ;)
    var btm = $(".tunnelshadow")[0];
    var tunnel = $(".tunnel")[0];
    function _refresh() {
        var wnd = Math.min(120, banner.clientHeight - tunnel.clientHeight);
        var scrolledfrac = window.scrollY / btm.offsetTop;
        if (scrolledfrac <= 1.4) {
            banner.style.top = (-wnd * scrolledfrac) + "px";
        }
    }
    $(document).scroll(_refresh);
    $(banner).ready(_refresh);
    $(window).resize(_refresh);
    $(".anisedoc_video").each(function(){
        this.style.visibility = "inherit";
        var cnt = this.textContent;
        var i = cnt.indexOf("|||");
        var vid = document.createElement("video");
        vid.preload = "metadata";
        vid.src = cnt.substring(0, i);
        vid.controls = true;
        var lbl = document.createElement("div");
        lbl.className = "caption";
        lbl.textContent = cnt.substring(i+3);
        this.innerHTML = "";
        this.appendChild(vid);
        this.appendChild(lbl);
        this._stop = function() {
            vid.pause();
        };
        this._isactive = function() {
            return !vid.paused;
        }
        this._registerloadhandler = function(h) {
            $(vid).on('loadedmetadata', h);
        }
        this._getoptimaldims = function() {
            return { width: vid.videoWidth, height: vid.videoHeight };
        }
    });
    $(".anisedoc_image").each(function(){
        var iimg = $(this).find("img");
        iimg.css("cursor", "pointer");
        iimg.click(function(){
            window.open(iimg[0].src, "_blank");
        });
        this._registerloadhandler = function(h) {
            iimg.on('load', h);
        }
    });
    $(".anisedoc_imagecarousel").each(function(){
        var self = this;
        new AniseDoc_ImageCarousel(this);
        this.style.visibility = "hidden";
        setTimeout(function(){ self.style.visibility = "inherit"; }, 2000);
    });
});
