# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Feature for gpg encryption.
"""

import base64

import anise.utils


def call_pgp(*args, raise_on_errors="unable to call gpg", **kwargs):
    """!
    Calls gpg with some arguments.
    """
    return anise.utils.basic.call(args, raise_on_errors=raise_on_errors, **kwargs)


def get_key(keyid, homedir=None, witharmor=True, withbase64=True):
    """!
    Returns a pgp key.
    @param keyid: The key id to look for.
    @param homedir: The pgp home directory.
    @param witharmor: If to apply an armor to the output.
    @param withbase64: If to apply additional base64 encoding to the output.
    @return: The key as bytes.
    """
    p = ["gpg"]
    if homedir:
        p += ["--homedir", homedir]
    if witharmor:
        p += ["--armor"]
    p += ["--export", keyid]
    r, (o, eo) = call_pgp(*p, decode=False, split_output_channels=True)
    bres = o
    if withbase64:
        bres = base64.b64encode(o)
    return bres
