# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.features
import anise.framework.projects

from anise.framework.projects import universe

projectdescriptioneditor = anise.framework.features.loadfeature("projectdescriptioneditor").featuremodule


HOOK_GET_DISTRIBUTABLES_PROVIDERS = anise.framework.features.Hook(
    hide=True, doc="This hook is called for collecting configuration guide helpers for distributables.")


@anise.framework.features.hookhandler(projectdescriptioneditor.HOOK_GET_FEATURE_ACTIONS)
class ProjectDescriptionEditorAssistant(projectdescriptioneditor.ProjectDescriptionEditorAssistant):

    def __init__(self):
        projectdescriptioneditor.ProjectDescriptionEditorAssistant.__init__(
            self, "Manage release distributables", "Specify which products to generate at release time. Those are "
                                                   "typically binary installation packages or source packages from "
                                                   "your project. They are typically generated for the project "
                                                   "homepage's download section.", "distributables")

    def getcustomactions(self, ais):
        return [projectdescriptioneditor.CustomAction("Add new distributable", self._add_distributable,
                                                      icon="userfeedback/add.png")]

    def _add_distributable(self, ais):
        impls = [x() for x in universe.gethooks(HOOK_GET_DISTRIBUTABLES_PROVIDERS)]
        implnames = [x.label for x in impls]
        i = universe.ui.userfeedback.choice("Please choose a distributable type.", implnames)
        if i is not None:
            r = impls[i].ask()
            if r is not None:
                name = universe.ui.userfeedback.input("Please enter a short description for this distributable.")
                if name is None:
                    return
                newlet = anise.framework.projects.IntermediateStructure.Let(
                    k="distributables.pool.addgroup",
                    sources=anise.framework.projects.IntermediateStructure.Value(
                        "files.TaskExecution",
                        task=anise.framework.projects.IntermediateStructure.Value("objectref", oref=impls[i].taskref),
                        source=anise.framework.projects.IntermediateStructure.Value("packages.RawPackage")),
                    name=anise.framework.projects.IntermediateStructure.Value.fromstring(name))
                for arg in r.args():
                    newlet.setarg(arg, r.getarg(arg))
                ais.addentry(newlet)

    def getentries(self, ais):
        r = []
        for e in ais.entries():
            if isinstance(e, anise.framework.projects.IntermediateStructure.Let) \
                    and e.key() == "distributables.pool.addgroup":
                r.append(e)
        return r

    def getentrylabel(self, ais, entry):
        return entry.getarg("name").value()
