# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.framework.features
import anise.framework.projects

from anise.framework.projects import universe

projectdescriptioneditor = anise.framework.features.loadfeature("projectdescriptioneditor").featuremodule


@anise.framework.features.hookhandler(projectdescriptioneditor.HOOK_GET_FEATURE_ACTIONS)
class ProjectDescriptionEditorAssistant(projectdescriptioneditor.ProjectDescriptionEditorAssistant):

    def __init__(self):
        projectdescriptioneditor.ProjectDescriptionEditorAssistant.__init__(
            self, "Manage licensing", "Specify how your project is licensed. This information is used by other parts, "
                                      "which generate content, like documentation or the project homepage.",
            "licensing")

    def getcustomactions(self, ais):
        return [projectdescriptioneditor.CustomAction("Set project license", self._set_license,
                                                      icon="userfeedback/preferences.png")]

    def _set_license(self, ais):
        lics = dir(universe.licensing)
        lics = [x for x in lics if x[0].upper() in "ABCDEFGHIJKLMNOPQRSTUVWXYZ"]
        lics = [x for x in lics if x[0].isupper()]
        lics = [x for x in lics if x != "BaseLicense"]
        clics = lics + ["Own custom license"]
        newlic = universe.ui.userfeedback.choice("Please choose a license.", clics)
        if newlic is None:
            return
        licargs = {}
        if newlic < len(lics):
            licclassname = "licensing." + lics[newlic]
        else:
            lname = universe.ui.userfeedback.input("Please specify the name for your license.",
                                                   "My Own Public License 1")
            if lname is None:
                return
            lfilename = universe.ui.userfeedback.input("Please specify the file, which contains the license text.",
                                                       "_meta/license.src")
            if lfilename is None:
                return
            licargs["name"] = anise.framework.projects.IntermediateStructure.Value.fromstring(lname)
            licargs["filename"] = anise.framework.projects.IntermediateStructure.Value.fromstring(lfilename)
            licclassname = "licensing.BaseLicense"
        licent = None
        for x in ais.entries():
            if isinstance(x, anise.framework.projects.IntermediateStructure.Value) and x.key() == "base.license":
                licent = x
                licent.clearargs()
                licent.setvtype(licclassname)
                break
        if licent is None:
            licent = anise.framework.projects.IntermediateStructure.Value(licclassname, k="base.license")
            ais.addentry(licent)
        for licarg in licargs:
            licent.setarg(licarg, licargs[licarg])

    def getentries(self, ais):
        r = []
        for e in ais.entries():
            if isinstance(e, anise.framework.projects.IntermediateStructure.Value) and e.key() == "base.license":
                r.append(e)
        return r

    def getentrylabel(self, ais, entry):
        name = entry.getarg("name")
        return "Current license: " + entry.vtype() + ((" / " + name.value()) if name else "")
