# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

"""
Feature for unit testing with pyunit.
"""

import anise.framework.features
import anise.utils

from anise.framework.projects import universe

testinggeneric = anise.framework.features.loadfeature("testing.generic").featuremodule


def pyunittest(testrun, testclass, testdir=None):
    """!
    Runs an pyunit test.
    @param testrun: A testing.internals.TestRun instance.
    @param testdir: The root directory where to look for tests.
    @param testclass: The name of the test class.
    """
    with anise.utils.basic.ChDir(universe.specialpaths.projectdir):
        testinggeneric.generictest(testrun, ["python3", "-m", "unittest", testclass, "-v", "-b", "-s", testdir or ""])


def pyunittests(testrun, testdir=None, filepattern="*.py"):
    """!
    Runs all pyunit tests in the project (or all which fulfill specified patterns). The discovery is done automatically.
    @param testrun: A testing.internals.TestRun instance.
    @param testdir: The root directory where to look for tests.
    @param filepattern: A file pattern describing which files to inspect.
    """
    with anise.utils.basic.ChDir(universe.specialpaths.projectdir):
        testinggeneric.generictest(testrun, ["python3", "-m", "unittest", "discover", "-v", "-b",
                                             "-s", testdir or "", "-p", filepattern])
