# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

## @package anise.framework.exceptions
## Exception subclasses


class AniseError(Exception):
    """!
    An error in the execution of Anise.
    """

    def __call__(self, *args):
        return self.__class__(*(self.args + args))


class BadInputError(AniseError):
    """!
    A bad value was given as some input.
    """
    pass


class BadFormatError(AniseError):
    """!
    Some input is wrongly formatted.
    """
    pass


class BadCommunicationError(AniseError):
    """!
    The communication with some other component failed.
    """
    pass


class ProcessExecutionFailedError(AniseError):
    """!
    The execution of an external process failed.
    """

    def __init__(self, text, output=None):
        if output:
            output = output.strip()
        if output == "":
            text += " (without output)"
        fulltext = ": ".join([x for x in [text, output] if x])
        AniseError.__init__(self, fulltext)
        self.output = output


class ImplementationError(AniseError):
    """!
    A software error in an implementation occurred.
    """
    pass


class InternalError(ImplementationError):
    """!
    An internal error in the anise code occurred.
    """
    pass


class NotImplementedError(InternalError):
    """!
    Some unimplemented functionality was called.
    """
    def __init__(self):
        InternalError.__init__(self, "Some unimplemented functionality was called.")


class RequirementsMissingError(AniseError):
    """!
    Something required is not in place.
    """
    pass


class UnexpectedSituationError(AniseError):
    """!
    An unexpected situation led to an error.
    """
    pass
