# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

## @package anise.test.anisetest
## Anise testing backend.

import os
import shutil
import unittest

import anise.framework.engine


class AniseTestCase(unittest.TestCase):

    def __init__(self, *a, projectdesc="raise Exception('test has not specified a projectdesc')", **kwa):
        unittest.TestCase.__init__(self, *a, **kwa)
        self.projectdesc = projectdesc

    def setUp(self):
        self.rootdir = "/tmp/anise.test." + str(os.getpid())
        if os.path.exists(self.rootdir):
            shutil.rmtree(self.rootdir)
        self.packagestoredir = self.rootdir + "/packagestore"
        self.mydir = self.rootdir + "/workdir"
        os.makedirs(self.mydir)
        os.makedirs(self.packagestoredir)
        self.projdescfile = self.mydir + "/_projectdesc"
        with open(self.projdescfile, "w") as f:
            f.write(self.projectdesc)

    def runanise(self, taskname, values=None, loadfeaturesfrom=None):
        _values = dict(values or {})
        _values["packagestore.path"] = self.packagestoredir
        _values["ui.force_mode"] = "TERMINAL"
        return anise.framework.engine.execute(taskname, self.projdescfile, self.mydir, _values,
                                              list(["builtin"] if (loadfeaturesfrom is None) else loadfeaturesfrom))

    def tearDown(self):
        if os.path.exists(self.rootdir):
            shutil.rmtree(self.rootdir)
