# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="foo" t="int" v="1" />
    <val k="bar" t="int" v="2" />
    <val k="sum" oref="myfeature.sum" />
</aniseprojectdesc>
"""

_myfeature = """
import anise.framework.features

from anise.framework.projects import universe


def sum():
    universe.myfeature.registerfoohook()
    for h in universe.gethooks("MYHOOK"):
        h()
    return universe.foo + universe.bar

def registerfoohook():
    def foohook():
        universe.foo = 37
    universe.addhook("MYHOOK", foohook)

@anise.framework.features.hookhandler("MYHOOK")
def barhook():
    universe.bar = 1300
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        anise.test.anisetest.AniseTestCase.__init__(self, *a, projectdesc=_projectdesc, **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.mydir + "/features")
        with open(self.mydir + "/features/myfeature.py", "w") as f:
            f.write(_myfeature)
        loadfeaturesfrom = ["builtin", self.mydir + "/features"]
        self.assertEqual(self.runanise("sum", loadfeaturesfrom=loadfeaturesfrom), 1337)
        self.assertEqual(self.runanise("sum", values={'foo': 3}, loadfeaturesfrom=loadfeaturesfrom), 1337)



