# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os

import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.GPLv3OrHigher" />
    <let k="base.pyscript">
        <val k="hook" oref="engine.HOOK_AFTER_PREPARATION"/>
        <val k="code">
def fct():
    def result():
        pyunit = universe.pyunit
        p = files.RemappedFilestructure(frommap=(
                    ('/baz/ab/a', '/foo/a'),
                    ('/baz/ab/b', '/foo/b'),
                    ('/x/x/d', '/bar/c'),
                    ('/y/c', '/bar/d'),
                    ('/z/z', '/foobar'),
                ), source=files.Filestructure("xx", relative=True)).path()
        pyunit.assertEqual("aa", utils.basic.readfromfile(p + "/baz/ab/a"))
        pyunit.assertEqual("bb", utils.basic.readfromfile(p + "/baz/ab/b"))
        pyunit.assertEqual("cc", utils.basic.readfromfile(p + "/x/x/d"))
        pyunit.assertEqual("dd", utils.basic.readfromfile(p + "/y/c"))
        pyunit.assertEqual("ee", utils.basic.readfromfile(p + "/z/z/e"))
        pyunit.assertEqual("ff", utils.basic.readfromfile(p + "/z/z/f"))
    universe.result = result
        </val>
    </let>
</aniseprojectdesc>
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        anise.test.anisetest.AniseTestCase.__init__(self, *a, projectdesc=_projectdesc, **kwa)

    def test_a(self):
        os.makedirs(self.mydir + "/xx/foo")
        os.makedirs(self.mydir + "/xx/bar")
        os.makedirs(self.mydir + "/xx/foobar")
        anise.utils.basic.writetofile(self.mydir + "/xx/foo/a", "aa")
        anise.utils.basic.writetofile(self.mydir + "/xx/foo/b", "bb")
        anise.utils.basic.writetofile(self.mydir + "/xx/bar/c", "cc")
        anise.utils.basic.writetofile(self.mydir + "/xx/bar/d", "dd")
        anise.utils.basic.writetofile(self.mydir + "/xx/foobar/e", "ee")
        anise.utils.basic.writetofile(self.mydir + "/xx/foobar/f", "ff")
        self.runanise("result", values={'pyunit': self})
