# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os
import tarfile

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <let k="base.set_basecolor">
        <val k="r" t="float" v="0.1" />
        <val k="g" t="float" v="0.1" />
        <val k="b" t="float" v="0.1" />
    </let>
    <val k="base.longdescription" v="mylongdescription" />
    <let k="documentation.has_developerdocumentation"/>
    <let k="distributables.pool.addgroup" name="source tarball">
        <val k="sources" t="files.TaskExecution">
            <val k="task" oref="packages.tarpackage"/>
            <val k="source" t="packages.RawPackage"/>
        </val>
    </let>
    <let k="releasing.tasks.add" destination="{0}">
        <val k="task" oref="filetransfer.localupload" />
        <val k="source" t="files.TaskExecution">
            <val k="task" oref="homepage.makehomepage" />
        </val>
    </let>
</aniseprojectdesc>
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path()
        anise.test.anisetest.AniseTestCase.__init__(self, *a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.writetofile(self.mydir + "/x", "aa")
        anise.utils.basic.writetofile(self.mydir + "/y", "bb")
        self.runanise("releasing.release")
        p = self.destination
        indexhtml = anise.utils.basic.readfromfile(p + "/myprojectname/index.html")
        self.assertTrue("myprojectname" in indexhtml)
        self.assertTrue("1.2" in indexhtml)
        self.assertTrue("stable" in indexhtml.lower())
        self.assertTrue("publicdomain" in indexhtml.lower())
        self.assertTrue("mysummary" in indexhtml)
        self.assertTrue("mylongdescription" in indexhtml)
        self.assertTrue("no dependencies" in indexhtml)
        self.assertTrue("documentation" in indexhtml)

        self.assertTrue(os.path.exists(p + "/myprojectname/developerdoc/index.html"))
        self.assertTrue(os.path.exists(p + "/myprojectname/developerdoc/pages.html"))
        self.assertTrue(os.path.exists(p + "/myprojectname/developerdoc/files.html"))
        self.assertTrue(os.path.exists(p + "/myprojectname/developerdoc/FULLREADME.html"))

        tar = tarfile.open(p + "/myprojectname/myprojectname-1.2.tgz", "r:gz")
        tarcnt = {}
        for tarinfo in tar:
            tarcnt[tarinfo.name] = tarinfo.size
        tar.close()

        self.assertEqual(tarcnt["myprojectname-1.2/x"], 2)
        self.assertEqual(tarcnt["myprojectname-1.2/y"], 2)

        self.assertGreater(tarcnt["myprojectname-1.2/README"], 100)
        self.assertGreater(tarcnt["myprojectname-1.2/LICENSE"], 100)
        self.assertGreater(tarcnt["myprojectname-1.2/README.pdf"], 100)
