# Copyright (C) 2014-2020, Josef Hahn
#
# This file is part of anise.
#
# anise is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# anise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with anise.  If not, see <http://www.gnu.org/licenses/>.

import os

import anise.framework.files
import anise.test.anisetest
import anise.utils


_projectdesc = """<?xml version="1.0" ?>
<aniseprojectdesc>
    <val k="name" v="myprojectname" />
    <val k="base.version" v="1.2" />
    <val k="base.summary" v="mysummary" />
    <val k="base.email" v="my@mail.address" />
    <val k="base.authors"><val v="John Doe" /></val>
    <val k="base.license" oref="licensing.PublicDomain" />
    <val k="base.maturity" oref="base.Maturity.ProductionStable" />
    <val k="base.longdescription" v="mylongdescription" />
</aniseprojectdesc>
"""

_myfeature = """
from anise.framework.projects import universe

def foo():
    return universe.build.autotools.build(source=universe.packages.RawPackage(), 
                                          configure_args=("--prefix", "/usr/loccal"))
"""


class Test(anise.test.anisetest.AniseTestCase):

    def __init__(self, *a, **kwa):
        self.destination = anise.framework.files.TempDir().path()
        anise.test.anisetest.AniseTestCase.__init__(self, *a, projectdesc=_projectdesc.format(self.destination), **kwa)

    def test_a(self):
        anise.utils.basic.mkdirs(self.rootdir + "/features")
        anise.utils.basic.writetofile(self.rootdir + "/features/myfeature.py", _myfeature)
        loadfeaturesfrom = ["builtin", self.rootdir + "/features"]

        os.makedirs(self.mydir+"/src")

        anise.utils.basic.writetofile(self.mydir + "/src/foo.c", """
#include <config.h>
#include <stdio.h>

int main (void)
{
    puts ("foo!");
    return 0;
}
        """)

        anise.utils.basic.writetofile(self.mydir + "/src/bar.c", """
#include <config.h>
#include <stdio.h>

int main (void)
{
    puts ("bar!");
    return 0;
}
        """)

        anise.utils.basic.writetofile(self.mydir + "/baz", """
        baz!
        """)

        anise.utils.basic.writetofile(self.mydir + "/Makefile.am", """
SUBDIRS = src
        """)

        anise.utils.basic.writetofile(self.mydir + "/src/Makefile.am", """
bin_PROGRAMS = foo bar
foo_SOURCES = foo.c
bar_SOURCES = bar.c
amfoo_dir = $(pkgdatadir)/mydata
amfoo__DATA = ../baz
        """)

        anise.utils.basic.writetofile(self.mydir + "/configure.ac", """
AC_INIT([amfoo], [1.0], [foo@example.org])
AM_INIT_AUTOMAKE([-Wall -Werror foreign])
AC_PROG_CC
AC_CONFIG_HEADERS([config.h])
AC_CONFIG_FILES([
 Makefile
 src/Makefile
])
AC_OUTPUT
        """)

        with anise.utils.basic.ChDir(anise.framework.files.TempDir().path()):
            mytool = self.runanise("myfeature.foo", loadfeaturesfrom=loadfeaturesfrom).path()
            r1, o1 = anise.utils.basic.call(mytool + "/usr/loccal/bin/foo")
            r2, o2 = anise.utils.basic.call(mytool + "/usr/loccal/bin/bar")
        self.assertEqual(o1, "foo!")
        self.assertEqual(o2, "bar!")
        o3 = anise.utils.basic.readfromfile(mytool + "/usr/loccal/share/amfoo/mydata/baz").strip()
        self.assertEqual(o3, "baz!")
